﻿using System;
using System.Windows;
using System.Text;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using ESRI.ArcGIS.Client.Symbols;
using System.Windows.Media;

namespace ESRI.ArcGIS.Silverlight.Shapefile
{
    /// <summary>
    /// The ShapeFileRecord class represents the contents of
    /// a shape record, which is of variable length.
    /// </summary>
    public class ShapeFileRecord
    {
        #region Private fields

        // 默认图形符号
        public static SimpleMarkerSymbol DEFAULT_MARKER_SYMBOL = null;
        public static SimpleLineSymbol DEFAULT_LINE_SYMBOL = null;
        public static SimpleFillSymbol DEFAULT_FILL_SYMBOL = null;

        // 记录头信息
        private int _recordNumber;
        private int _contentLength;

        // 图形类型
        private int _shapeType;

        // 图形外包矩形角点坐标
        private double _xMin;
        private double _yMin;
        private double _xMax;
        private double _yMax;

        // Part 索引和点数组
        private Collection<int> _parts = new Collection<int>();
        private Collection<Point> _points = new Collection<Point>();

        // 属性信息
        private IDictionary<string, object> _attributes;

        #endregion

        #region Constructor

        public ShapeFileRecord()
        {
            DEFAULT_MARKER_SYMBOL = new SimpleMarkerSymbol()
            {
                Style = SimpleMarkerSymbol.SimpleMarkerStyle.Circle,
                Color = new SolidColorBrush(Colors.Red)
            };

            DEFAULT_LINE_SYMBOL = new SimpleLineSymbol()
            {
                Color = new SolidColorBrush(Colors.Red),
                Style = SimpleLineSymbol.LineStyle.Solid,
                Width = 2
            };

            DEFAULT_FILL_SYMBOL = new SimpleFillSymbol()
            {
                Fill = new SolidColorBrush(Colors.Red),
                BorderBrush = new SolidColorBrush(Colors.White),
                BorderThickness = 2
            };
        }

        #endregion

        #region Properties

        /// <summary>
        /// Indicates the record number (or index) which starts at 1.
        /// </summary>
        public int RecordNumber
        {
            get
            {
                return this._recordNumber;
            }
            set
            {
                this._recordNumber = value;
            }
        }

        /// <summary>
        /// Specifies the length of this shape record in 16-bit words.
        /// </summary>
        public int ContentLength
        {
            get
            {
                return this._contentLength;
            }
            set
            {
                this._contentLength = value;
            }
        }

        /// <summary>
        /// Specifies the shape type for this record.
        /// </summary>
        public int ShapeType
        {
            get
            {
                return this._shapeType;
            }
            set
            {
                this._shapeType = value;
            }
        }

        /// <summary>
        /// Indicates the minimum x-position of the bounding
        /// box for the shape (expressed in degrees longitude).
        /// </summary>
        public double XMin
        {
            get
            {
                return this._xMin;
            }
            set
            {
                this._xMin = value;
            }
        }

        /// <summary>
        /// Indicates the minimum y-position of the bounding
        /// box for the shape (expressed in degrees latitude).
        /// </summary>
        public double YMin
        {
            get
            {
                return this._yMin;
            }
            set
            {
                this._yMin = value;
            }
        }

        /// <summary>
        /// Indicates the maximum x-position of the bounding
        /// box for the shape (expressed in degrees longitude).
        /// </summary>
        public double XMax
        {
            get
            {
                return this._xMax;
            }
            set
            {
                this._xMax = value;
            }
        }

        /// <summary>
        /// Indicates the maximum y-position of the bounding
        /// box for the shape (expressed in degrees latitude).
        /// </summary>
        public double YMax
        {
            get
            {
                return this._yMax;
            }
            set
            {
                this._yMax = value;
            }
        }

        /// <summary>
        /// Indicates the number of parts for this shape.
        /// A part is a connected set of points, analogous to
        /// a PathFigure in WPF.
        /// </summary>
        public int NumberOfParts
        {
            get
            {
                return this._parts.Count;
            }
        }

        /// <summary>
        /// Specifies the total number of points defining
        /// this shape record.
        /// </summary>
        public int NumberOfPoints
        {
            get
            {
                return this._points.Count;
            }
        }

        /// <summary>      
        /// A collection of indices for the points array.
        /// Each index identifies the starting point of the
        /// corresponding part (or PathFigure using WPF
        /// terminology).
        /// </summary>
        public Collection<int> Parts
        {
            get
            {
                return this._parts;
            }
        }

        /// <summary>
        /// A collection of all of the points defining the
        /// shape record.
        /// </summary>
        public Collection<Point> Points
        {
            get
            {
                return this._points;
            }
        }

        /// <summary>
        /// Access the (dBASE) attribute values associated
        /// with this shape record.
        /// </summary>
        public IDictionary<string, object> Attributes
        {
            get
            {
                return this._attributes;
            }
            set
            {
                this._attributes = value;
            }
        }

        #endregion

        #region Public methods

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendFormat( "ShapeFileRecord: RecordNumber={0}, ContentLength={1}, ShapeType={2}",
                this._recordNumber, this._contentLength, this._shapeType );

            return sb.ToString();
        }

        /// <summary>
        /// 将Shapefile中的记录转换为Graphic图形对象
        /// </summary>
        /// <param name="symbol">图形的符号，为null时将使用默认符号</param>
        /// <returns>Graphic图形对象</returns>
        public Graphic ToGraphic(Symbol symbol = null)
        {
            Graphic graphic = new Graphic();

            //将要素属性赋给Graphic对象
            foreach (var item in this.Attributes)
            {
                graphic.Attributes.Add(item.Key, item.Value);
            }

            //将要素的图形赋给Graphic对象
            switch (this.ShapeType)
            {
                case (int)EsriShapeType.NullShape:
                    break;
                case (int)EsriShapeType.Multipoint:
                    graphic.Geometry = GetMultiPoint(this);
                    graphic.Symbol = symbol == null ? DEFAULT_MARKER_SYMBOL : symbol;
                    break;
                case (int)EsriShapeType.Point:
                    graphic.Geometry = GetPoint(this);
                    graphic.Symbol = symbol == null ? DEFAULT_MARKER_SYMBOL : symbol;
                    break;
                case (int)EsriShapeType.Polygon:
                    graphic.Geometry = GetPolygon(this);
                    graphic.Symbol = symbol == null ? DEFAULT_FILL_SYMBOL : symbol;
                    break;
                case (int)EsriShapeType.PolyLine:
                    graphic.Geometry = GetPolyline(this);
                    graphic.Symbol = symbol == null ? DEFAULT_LINE_SYMBOL : symbol;
                    break;
                default:
                    break;
            }

            return graphic;
        }

        private ESRI.ArcGIS.Client.Geometry.Geometry GetPolyline(ShapeFileRecord record)
        {
            Polyline line = new Polyline();
            for (int i = 0; i < record.NumberOfParts; i++)
            {
                // 确定起止索引
                int start = record.Parts[i];
                int end;
                if (record.NumberOfParts > 1 && i != (record.NumberOfParts - 1))
                    end = record.Parts[i + 1];
                else
                    end = record.NumberOfPoints;

                ESRI.ArcGIS.Client.Geometry.PointCollection points = new ESRI.ArcGIS.Client.Geometry.PointCollection();
                // Add line segments to the polyline
                for (int j = start; j < end; j++)
                {
                    System.Windows.Point point = record.Points[j];
                    points.Add(new MapPoint(point.X, point.Y));
                }

                line.Paths.Add(points);
            }

            return line;
        }

        private ESRI.ArcGIS.Client.Geometry.Geometry GetPolygon(ShapeFileRecord record)
        {
            Polygon polygon = new Polygon();
            for (int i = 0; i < record.NumberOfParts; i++)
            {
                // Determine the starting index and the end index
                // into the points array that defines the figure.
                int start = record.Parts[i];
                int end;
                if (record.NumberOfParts > 1 && i != (record.NumberOfParts - 1))
                    end = record.Parts[i + 1];
                else
                    end = record.NumberOfPoints;

                ESRI.ArcGIS.Client.Geometry.PointCollection points = new ESRI.ArcGIS.Client.Geometry.PointCollection();
                // Add line segments to the polyline
                for (int j = start; j < end; j++)
                {
                    System.Windows.Point point = record.Points[j];
                    points.Add(new MapPoint(point.X, point.Y));
                }

                polygon.Rings.Add(points);
            }

            return polygon;
        }

        private ESRI.ArcGIS.Client.Geometry.Geometry GetPoint(ShapeFileRecord record)
        {
            MapPoint point = new MapPoint();
            point.X = record.Points[0].X;
            point.Y = record.Points[0].Y;

            return point;
        }

        private ESRI.ArcGIS.Client.Geometry.Geometry GetMultiPoint(ShapeFileRecord record)
        {
            MultiPoint points = new MultiPoint();
            for (int i = 0; i < record.Points.Count; i++)
            {
                System.Windows.Point point = record.Points[i];
                points.Points.Add(new MapPoint(point.X, point.Y));
            }

            return points;
        }

        #endregion
    }
}
